@@ -3,7 +3,6 @@ package URI::Fast::Test;
 use strict;
 use warnings;
 use Test2::V0;
-use URI::Fast qw(uri);
 
 use parent 'Exporter';
 
@@ -24,22 +23,23 @@ sub export_uri {
 
 sub is_same_uri {
   my ($got, $expected, $msg) = @_;
-  is export_uri(uri("$got")), export_uri(uri("$expected")), $msg;
+  is export_uri($got), export_uri($expected), $msg;
 }
 
 sub isnt_same_uri {
   my ($got, $expected, $msg) = @_;
-  isnt export_uri(uri("$got")), export_uri(uri("$expected")), $msg;
+  isnt export_uri($got), export_uri($expected), $msg;
 }
 
 1;
 
 =head1 NAME
 
-URI::Fast::Test - Unit test comparisons for URI::Fast objects
+URI::Fast::Test
 
 =head1 SYNOPSIS
 
+  use URI::Fast qw(uri);
   use URI::Fast::Test;
 
   is_same_uri uri($got), uri($expected), 'got expected uri';
@@ -51,14 +51,12 @@ URI::Fast::Test - Unit test comparisons for URI::Fast objects
 =head2 is_same_uri
 
 Builds a nested structure of uri components for comparison with Test2's deep
-comparison using C<is>. The test subjects may be either L<URI::Fast> objects
-or strings, in which case they will be parsed into L<URI::Fast> objects.
+comparison using C<is>.
 
 =head2 isnt_same_uri
 
 Builds a nested structure of uri components for comparison with Test2's deep
-comparison using C<isnt>. The test subjects may be either L<URI::Fast> objects
-or strings, in which case they will be parsed into L<URI::Fast> objects.
+comparison using C<isnt>.
 
 =head1 SUBROUTINES
 
@@ -78,14 +76,4 @@ functions. The return value's structure is:
     frag   => $uri->frag,
   }
 
-=head1 AUTHOR
-
-Jeff Ober <sysread@fastmail.fm>
-
-=head1 COPYRIGHT AND LICENSE
-
-This software is copyright (c) 2018 by Jeff Ober. This is free software; you
-can redistribute it and/or modify it under the same terms as the Perl 5
-programming language system itself.
-
 =cut
